@@ -4,13 +4,14 @@ import shlex
 import tempfile
 import time
 
-from typing import Any, Callable, cast, Dict, List, Optional, Union, Tuple, TYPE_CHECKING
+from typing import Any, Callable, cast, Dict, List, Optional, Tuple, TYPE_CHECKING, Union
 
 from ..async_executor import CriticalTask
 from ..collections import EMPTY_DICT
 from ..dcs import Leader, Member, RemoteMember
 from ..psycopg import quote_ident, quote_literal
 from ..utils import deep_compare, unquote
+from .misc import PostgresqlState
 
 if TYPE_CHECKING:  # pragma: no cover
     from . import Postgresql
@@ -114,7 +115,7 @@ class Bootstrap(object):
         return user_options
 
     def _initdb(self, config: Any) -> bool:
-        self._postgresql.set_state('initializing new cluster')
+        self._postgresql.set_state(PostgresqlState.INITDB)
         not_allowed_options = ('pgdata', 'nosync', 'pwfile', 'sync-only', 'version')
 
         def error_handler(e: str) -> None:
@@ -138,7 +139,7 @@ class Bootstrap(object):
         if ret:
             self._postgresql.configure_server_parameters()
         else:
-            self._postgresql.set_state('initdb failed')
+            self._postgresql.set_state(PostgresqlState.INITDB_FAILED)
         return ret
 
     def _post_restore(self) -> None:
@@ -186,7 +187,7 @@ class Bootstrap(object):
         :returns: ``True`` if the bootstrap was successful, i.e. the execution of the custom ``command`` from *config*
             exited with code ``0``, ``False`` otherwise.
         """
-        self._postgresql.set_state('running custom bootstrap script')
+        self._postgresql.set_state(PostgresqlState.CUSTOM_BOOTSTRAP)
         params = [] if config.get('no_params') else ['--scope=' + self._postgresql.scope,
                                                      '--datadir=' + self._postgresql.data_dir]
         # Add custom parameters specified by the user
@@ -196,7 +197,7 @@ class Bootstrap(object):
         try:
             logger.info('Running custom bootstrap script: %s', config['command'])
             if self._postgresql.cancellable.call(shlex.split(config['command']) + params) != 0:
-                self._postgresql.set_state('custom bootstrap failed')
+                self._postgresql.set_state(PostgresqlState.CUSTOM_BOOTSTRAP_FAILED)
                 return False
         except Exception:
             logger.exception('Exception during custom bootstrap')
@@ -234,14 +235,15 @@ class Bootstrap(object):
                 return False
         return True
 
-    def create_replica(self, clone_member: Union[Leader, Member, None]) -> Optional[int]:
+    def create_replica(self, clone_member: Union[Leader, Member, None],
+                       clone_from_leader: bool = False) -> Optional[int]:
         """
             create the replica according to the replica_method
             defined by the user.  this is a list, so we need to
             loop through all methods the user supplies
         """
 
-        self._postgresql.set_state('creating replica')
+        self._postgresql.set_state(PostgresqlState.CREATING_REPLICA)
         self._postgresql.schedule_sanity_checks_after_pause()
 
         is_remote_member = isinstance(clone_member, RemoteMember)
@@ -252,6 +254,10 @@ class Bootstrap(object):
         replica_methods = (clone_member.create_replica_methods if is_remote_member
                            else self._postgresql.create_replica_methods) or ['basebackup']
 
+        # If '--from-leader' parameter is set when reinit, always use basebackup
+        if clone_from_leader:
+            replica_methods = ['basebackup']
+
         if clone_member and clone_member.conn_url:
             r = clone_member.conn_kwargs(self._postgresql.config.replication)
             # add the credentials to connect to the replica origin to pgpass.
@@ -301,7 +307,7 @@ class Bootstrap(object):
                                           "datadir": self._postgresql.data_dir,
                                           "connstring": connstring})
                 else:
-                    for param in ('no_params', 'no_master', 'no_leader', 'keep_data'):
+                    for param in ('no_params', 'no_leader', 'keep_data'):
                         method_config.pop(param, None)
                 params = ["--{0}={1}".format(arg, val) for arg, val in method_config.items()]
                 try:
@@ -318,7 +324,11 @@ class Bootstrap(object):
                     logger.exception('Error creating replica using method %s', replica_method)
                     ret = 1
 
-        self._postgresql.set_state('stopped')
+                # replica creation method failed, clean up data directory if configuration allows
+                if not method_config.get('keep_data', False) and not self._postgresql.data_directory_empty():
+                    self._postgresql.remove_data_directory()
+
+        self._postgresql.set_state(PostgresqlState.STOPPED)
         return ret
 
     def basebackup(self, conn_url: str, env: Dict[str, str], options: Dict[str, Any]) -> Optional[int]:
@@ -332,6 +342,13 @@ class Bootstrap(object):
         not_allowed_options = ('pgdata', 'format', 'wal-method', 'xlog-method', 'gzip',
                                'version', 'compress', 'dbname', 'host', 'port', 'username', 'password')
         user_options = self.process_user_options('basebackup', options, not_allowed_options, logger.error)
+        cmd = [
+            self._postgresql.pgcommand("pg_basebackup"),
+            "--pgdata=" + self._postgresql.data_dir,
+            "-X",
+            "stream",
+            "--dbname=" + conn_url,
+        ] + user_options
 
         for bbfailures in range(0, maxfailures):
             if self._postgresql.cancellable.is_cancelled:
@@ -339,9 +356,8 @@ class Bootstrap(object):
             if not self._postgresql.data_directory_empty():
                 self._postgresql.remove_data_directory()
             try:
-                ret = self._postgresql.cancellable.call([self._postgresql.pgcommand('pg_basebackup'),
-                                                         '--pgdata=' + self._postgresql.data_dir, '-X', 'stream',
-                                                         '--dbname=' + conn_url] + user_options, env=env)
+                logger.debug('calling: %r', cmd)
+                ret = self._postgresql.cancellable.call(cmd, env=env)
                 if ret == 0:
                     break
                 else:
@@ -353,10 +369,13 @@ class Bootstrap(object):
             if bbfailures < maxfailures - 1:
                 logger.warning('Trying again in 5 seconds')
                 time.sleep(5)
+            elif not self._postgresql.data_directory_empty():
+                # pg_basebackup failed, clean up data directory
+                self._postgresql.remove_data_directory()
 
         return ret
 
-    def clone(self, clone_member: Union[Leader, Member, None]) -> bool:
+    def clone(self, clone_member: Union[Leader, Member, None], clone_from_leader: bool = False) -> bool:
         """
              - initialize the replica from an existing member (primary or replica)
              - initialize the replica using the replica creation method that
@@ -364,7 +383,7 @@ class Bootstrap(object):
                base backup)
         """
 
-        ret = self.create_replica(clone_member) == 0
+        ret = self.create_replica(clone_member, clone_from_leader) == 0
         if ret:
             self._post_restore()
         return ret
@@ -440,11 +459,8 @@ END;$$""".format(f, quote_ident(rewind['username'], postgresql.connection()))
                         postgresql.query(sql)
 
                 if config.get('users'):
-                    logger.warning('User creation via "bootstrap.users" will be removed in v4.0.0')
-
-                for name, value in (config.get('users') or EMPTY_DICT).items():
-                    if all(name != a.get('username') for a in (superuser, replication, rewind)):
-                        self.create_or_update_role(name, value.get('password'), value.get('options', []))
+                    logger.error('User creation is not be supported starting from v4.0.0. '
+                                 'Please use "bootstrap.post_bootstrap" script to create users.')
 
                 # We were doing a custom bootstrap instead of running initdb, therefore we opened trust
                 # access from certain addresses to be able to reach cluster and change password
