@@ -3,7 +3,7 @@ import abc
 
 from typing import Any, Dict, Optional
 
-from patroni.utils import parse_int, parse_bool
+from patroni.utils import parse_bool, parse_int
 
 
 class Tags(abc.ABC):
@@ -29,6 +29,7 @@ class Tags(abc.ABC):
             they all are boolean values that default to disabled.
             However ``nofailover`` tag is always returned if ``failover_priority`` tag is defined. In this case, we need
             both values to see if they are contradictory and the ``nofailover`` value should be used.
+            The same rule applies for ``nosync`` and ``sync_priority`` tags.
 
         :returns: a dictionary of tags set for this node. The key is the tag name, and the value is the corresponding
             tag value.
@@ -36,7 +37,8 @@ class Tags(abc.ABC):
         return {tag: value for tag, value in tags.items()
                 if any((tag not in ('clonefrom', 'nofailover', 'noloadbalance', 'nosync', 'nostream'),
                         value,
-                        tag == 'nofailover' and 'failover_priority' in tags))}
+                        tag == 'nofailover' and 'failover_priority' in tags,
+                        tag == 'nosync' and 'sync_priority' in tags))}
 
     @property
     @abc.abstractmethod
@@ -52,31 +54,49 @@ class Tags(abc.ABC):
         """``True`` if ``clonefrom`` tag is ``True``, else ``False``."""
         return self.tags.get('clonefrom', False)
 
+    def _priority_tag(self, bool_name: str, priority_name: str) -> int:
+        """Common logic for obtaining the value of a priority tag from ``tags`` if defined.
+
+        If boolean tag is defined as ``True``, this will return ``0``. Otherwise, it will return the value of
+        the respective priority tag, defaulting to ``1`` if it's not defined or invalid.
+
+        :param bool_name: name of the boolean tag (``nofailover``. ``nosync``).
+        :param priority_name: name of the priority tag (``failover_priority``, ``sync_priority``).
+
+        :returns: integer value based on the defined tags.
+        """
+        from_tags = self.tags.get(bool_name)
+        priority = parse_int(self.tags.get(priority_name))
+        priority = 1 if priority is None else priority
+        return 0 if from_tags else priority
+
+    def _bool_tag(self, bool_name: str, priority_name: str) -> bool:
+        """Common logic for obtaining the value of a boolean tag from ``tags`` if defined.
+
+        If boolean tag is not defined, this methods returns ``True`` if priority tag is non-positive,
+        ``False`` otherwise.
+
+        :param bool_name: name of the boolean tag (``nofailover``. ``nosync``).
+        :param priority_name: name of the priority tag (``failover_priority``, ``sync_priority``).
+
+        :returns: boolean value based on the defined tags.
+        """
+        from_tags = self.tags.get(bool_name)
+        if from_tags is not None:
+            # Value of bool tag takes precedence over priority tag
+            return bool(from_tags)
+        priority = parse_int(self.tags.get(priority_name))
+        return priority is not None and priority <= 0
+
     @property
     def nofailover(self) -> bool:
-        """Common logic for obtaining the value of ``nofailover`` from ``tags`` if defined.
-
-        If ``nofailover`` is not defined, this methods returns ``True`` if ``failover_priority`` is non-positive,
-        ``False`` otherwise.
-        """
-        from_tags = self.tags.get('nofailover')
-        if from_tags is not None:
-            # Value of `nofailover` takes precedence over `failover_priority`
-            return bool(from_tags)
-        failover_priority = parse_int(self.tags.get('failover_priority'))
-        return failover_priority is not None and failover_priority <= 0
+        """``True`` if node configuration doesn't allow it to become primary, ``False`` otherwise."""
+        return self._bool_tag('nofailover', 'failover_priority')
 
     @property
     def failover_priority(self) -> int:
-        """Common logic for obtaining the value of ``failover_priority`` from ``tags`` if defined.
-
-        If ``nofailover`` is defined as ``True``, this will return ``0``. Otherwise, it will return the value of
-        ``failover_priority``, defaulting to ``1`` if it's not defined or invalid.
-        """
-        from_tags = self.tags.get('nofailover')
-        failover_priority = parse_int(self.tags.get('failover_priority'))
-        failover_priority = 1 if failover_priority is None else failover_priority
-        return 0 if from_tags else failover_priority
+        """Value of ``failover_priority`` from ``tags`` if defined, otherwise derived from ``nofailover``."""
+        return self._priority_tag('nofailover', 'failover_priority')
 
     @property
     def noloadbalance(self) -> bool:
@@ -85,8 +105,13 @@ class Tags(abc.ABC):
 
     @property
     def nosync(self) -> bool:
-        """``True`` if ``nosync`` is ``True``, else ``False``."""
-        return bool(self.tags.get('nosync', False))
+        """``True`` if node configuration doesn't allow it to become synchronous, ``False`` otherwise."""
+        return self._bool_tag('nosync', 'sync_priority')
+
+    @property
+    def sync_priority(self) -> int:
+        """Value of ``sync_priority`` from ``tags`` if defined, otherwise derived from ``nosync``."""
+        return self._priority_tag('nosync', 'sync_priority')
 
     @property
     def replicatefrom(self) -> Optional[str]:
