@@ -1008,7 +1008,8 @@ schema = Schema({
         Optional("file_num"): int,
         Optional("file_size"): int,
         Optional("mode"): IntValidator(min=0, max=511, expected_type=int, raise_assert=True),
-        Optional("loggers"): dict
+        Optional("loggers"): dict,
+        Optional("deduplicate_heartbeat_logs"): bool
     },
     Optional("ctl"): {
         Optional("insecure"): bool,
@@ -1035,7 +1036,9 @@ schema = Schema({
         Optional("allowlist_include_members"): bool,
         Optional("http_extra_headers"): dict,
         Optional("https_extra_headers"): dict,
-        Optional("request_queue_size"): IntValidator(min=0, max=4096, expected_type=int, raise_assert=True)
+        Optional("request_queue_size"): IntValidator(min=0, max=4096, expected_type=int, raise_assert=True),
+        Optional("server_tokens"): EnumValidator(('minimal', 'productonly', 'original'),
+                                                 case_sensitive=False, raise_assert=True)
     },
     Optional("bootstrap"): {
         "dcs": {
@@ -1142,6 +1145,7 @@ schema = Schema({
             Optional("ports"): [{"name": str, "port": IntValidator(max=65535, expected_type=int, raise_assert=True)}],
             Optional("cacert"): str,
             Optional("retriable_http_codes"): Or(int, [int]),
+            Optional("bootstrap_labels"): dict,
         },
     }),
     Optional("citus"): {
@@ -1189,7 +1193,10 @@ schema = Schema({
         Optional("clonefrom"): bool,
         Optional("noloadbalance"): bool,
         Optional("replicatefrom"): str,
-        Optional("nosync"): bool,
+        AtMostOne("nosync", "sync_priority"): Case({
+            "nosync": bool,
+            "sync_priority": IntValidator(min=0, expected_type=int, raise_assert=True),
+        }),
         Optional("nostream"): bool
     }
 })
