@@ -1,8 +1,9 @@
 import unittest
 
-from mock import Mock, patch
-from patroni.async_executor import AsyncExecutor, CriticalTask
 from threading import Thread
+from unittest.mock import Mock, patch
+
+from patroni.async_executor import AsyncExecutor, CriticalTask
 
 
 class TestAsyncExecutor(unittest.TestCase):
