@@ -4,6 +4,7 @@ import re
 import stat
 import subprocess
 import time
+import unittest
 
 from copy import deepcopy
 from pathlib import Path
@@ -20,10 +21,11 @@ from patroni.collections import CaseInsensitiveDict, CaseInsensitiveSet
 from patroni.dcs import RemoteMember
 from patroni.exceptions import PatroniException, PostgresConnectionException
 from patroni.file_perm import pg_perm
-from patroni.postgresql import Postgresql, STATE_NO_RESPONSE, STATE_REJECT
+from patroni.postgresql import PgIsReadyStatus, Postgresql
 from patroni.postgresql.bootstrap import Bootstrap
 from patroni.postgresql.callback_executor import CallbackAction
 from patroni.postgresql.config import _false_validator, get_param_diff
+from patroni.postgresql.misc import PostgresqlRole, PostgresqlState
 from patroni.postgresql.postmaster import PostmasterProcess
 from patroni.postgresql.validator import _get_postgres_guc_validators, _load_postgres_gucs_validators, \
     _read_postgres_gucs_validators_file, Bool, Enum, EnumBool, Integer, InvalidGucValidatorsFile, \
@@ -161,7 +163,7 @@ class TestPostgresql(BaseTestPostgresql):
     @patch.object(Postgresql, 'pg_isready')
     @patch('patroni.postgresql.polling_loop', Mock(return_value=range(1)))
     def test_wait_for_port_open(self, mock_pg_isready):
-        mock_pg_isready.return_value = STATE_NO_RESPONSE
+        mock_pg_isready.return_value = PgIsReadyStatus.NO_RESPONSE
         mock_postmaster = MockPostmaster()
         mock_postmaster.is_running.return_value = None
 
@@ -241,7 +243,7 @@ class TestPostgresql(BaseTestPostgresql):
     @patch('time.sleep', Mock())
     @patch.object(Postgresql, 'is_running', MockPostmaster)
     @patch.object(Postgresql, '_wait_for_connection_close', Mock())
-    @patch.object(Postgresql, 'latest_checkpoint_location', Mock(return_value='7'))
+    @patch.object(Postgresql, 'latest_checkpoint_locations', Mock(return_value=(7, 7)))
     def test__do_stop(self):
         mock_callback = Mock()
         with patch.object(Postgresql, 'controldata',
@@ -259,7 +261,7 @@ class TestPostgresql(BaseTestPostgresql):
     def test_restart(self):
         self.p.start = Mock(return_value=False)
         self.assertFalse(self.p.restart())
-        self.assertEqual(self.p.state, 'restart failed (restarting)')
+        self.assertEqual(self.p.state, PostgresqlState.RESTART_FAILED)
 
     @patch('os.chmod', Mock())
     @patch('builtins.open', MagicMock())
@@ -380,7 +382,7 @@ class TestPostgresql(BaseTestPostgresql):
     @patch.object(MockCursor, 'execute', Mock(side_effect=psycopg.OperationalError))
     def test__query(self):
         self.assertRaises(PostgresConnectionException, self.p._query, 'blabla')
-        self.p._state = 'restarting'
+        self.p._state = PostgresqlState.RESTARTING
         self.assertRaises(RetryFailedError, self.p._query, 'blabla')
 
     def test_query(self):
@@ -388,7 +390,7 @@ class TestPostgresql(BaseTestPostgresql):
         self.assertRaises(PostgresConnectionException, self.p.query, 'RetryFailedError')
         self.assertRaises(psycopg.ProgrammingError, self.p.query, 'blabla')
 
-    @patch.object(Postgresql, 'pg_isready', Mock(return_value=STATE_REJECT))
+    @patch.object(Postgresql, 'pg_isready', Mock(return_value=PgIsReadyStatus.REJECT))
     def test_is_primary(self):
         self.assertTrue(self.p.is_primary())
         self.p.reset_cluster_info_state(None)
@@ -399,13 +401,13 @@ class TestPostgresql(BaseTestPostgresql):
                                                                 'Latest checkpoint location': '0/1ADBC18',
                                                                 "Latest checkpoint's TimeLineID": '1'}))
     @patch('subprocess.Popen')
-    def test_latest_checkpoint_location(self, mock_popen):
+    def test_latest_checkpoint_locations(self, mock_popen):
         mock_popen.return_value.communicate.return_value = (None, None)
-        self.assertEqual(self.p.latest_checkpoint_location(), 28163096)
+        self.assertEqual(self.p.latest_checkpoint_locations(), (28163096, 28163096))
         with patch.object(Postgresql, 'controldata', Mock(return_value={'Database cluster state': 'shut down',
                                                                         'Latest checkpoint location': 'k/1ADBC18',
                                                                         "Latest checkpoint's TimeLineID": '1'})):
-            self.assertIsNone(self.p.latest_checkpoint_location())
+            self.assertEqual(self.p.latest_checkpoint_locations(), (None, None))
         # 9.3 and 9.4 format
         mock_popen.return_value.communicate.side_effect = [
             (b'rmgr: XLOG        len (rec/tot):     72/   104, tx:          0, lsn: 0/01ADBC18, prev 0/01ADBBB8, '
@@ -413,14 +415,14 @@ class TestPostgresql(BaseTestPostgresql):
              + b' 1; offset 0; oldest xid 715 in DB 1; oldest multi 1 in DB 1; oldest running xid 0; shutdown', None),
             (b'rmgr: Transaction len (rec/tot):     64/    96, tx:        726, lsn: 0/01ADBBB8, prev 0/01ADBB70, '
              + b'bkp: 0000, desc: commit: 2021-02-26 11:19:37.900918 CET; inval msgs: catcache 11 catcache 10', None)]
-        self.assertEqual(self.p.latest_checkpoint_location(), 28163096)
+        self.assertEqual(self.p.latest_checkpoint_locations(), (28163096, 28163096))
         mock_popen.return_value.communicate.side_effect = [
             (b'rmgr: XLOG        len (rec/tot):     72/   104, tx:          0, lsn: 0/01ADBC18, prev 0/01ADBBB8, '
              + b'bkp: 0000, desc: checkpoint: redo 0/1ADBC18; tli 1; prev tli 1; fpw true; xid 0/727; oid 16386; multi'
              + b' 1; offset 0; oldest xid 715 in DB 1; oldest multi 1 in DB 1; oldest running xid 0; shutdown', None),
             (b'rmgr: XLOG        len (rec/tot):      0/    32, tx:          0, lsn: 0/01ADBBB8, prev 0/01ADBBA0, '
              + b'bkp: 0000, desc: xlog switch ', None)]
-        self.assertEqual(self.p.latest_checkpoint_location(), 28163000)
+        self.assertEqual(self.p.latest_checkpoint_locations(), (28163096, 28163000))
         # 9.5+ format
         mock_popen.return_value.communicate.side_effect = [
             (b'rmgr: XLOG        len (rec/tot):    114/   114, tx:          0, lsn: 0/01ADBC18, prev 0/018260F8, '
@@ -429,7 +431,7 @@ class TestPostgresql(BaseTestPostgresql):
              + b' oldest running xid 0; shutdown', None),
             (b'rmgr: XLOG        len (rec/tot):     24/    24, tx:          0, lsn: 0/018260F8, prev 0/01826080, '
              + b'desc: SWITCH ', None)]
-        self.assertEqual(self.p.latest_checkpoint_location(), 25321720)
+        self.assertEqual(self.p.latest_checkpoint_locations(), (28163096, 25321720))
 
     def test_reload(self):
         self.assertTrue(self.p.reload())
@@ -447,7 +449,7 @@ class TestPostgresql(BaseTestPostgresql):
         task = CriticalTask()
         self.assertTrue(self.p.promote(0, task))
 
-        self.p.set_role('replica')
+        self.p.set_role(PostgresqlRole.REPLICA)
         self.p.config._config['pre_promote'] = 'test'
         with patch('patroni.postgresql.cancellable.CancellableSubprocess.is_cancelled', PropertyMock(return_value=1)):
             self.assertFalse(self.p.promote(0, task))
@@ -459,7 +461,7 @@ class TestPostgresql(BaseTestPostgresql):
         self.assertFalse(self.p.promote(0, task))
 
     def test_timeline_wal_position(self):
-        self.assertEqual(self.p.timeline_wal_position(), (1, 2, 1))
+        self.assertEqual(self.p.timeline_wal_position(), (1, 2, 1, 1, 1))
         Thread(target=self.p.timeline_wal_position).start()
 
     @patch.object(PostmasterProcess, 'from_pidfile')
@@ -481,7 +483,7 @@ class TestPostgresql(BaseTestPostgresql):
 
     @patch('shlex.split', Mock(side_effect=OSError))
     def test_call_nowait(self):
-        self.p.set_role('replica')
+        self.p.set_role(PostgresqlRole.REPLICA)
         self.assertIsNone(self.p.call_nowait(CallbackAction.ON_START))
         self.p.bootstrapping = True
         self.assertIsNone(self.p.call_nowait(CallbackAction.ON_START))
@@ -509,7 +511,7 @@ class TestPostgresql(BaseTestPostgresql):
     @patch('os.path.exists', Mock(return_value=True))
     @patch.object(Postgresql, 'controldata', Mock())
     def test_get_postgres_role_from_data_directory(self):
-        self.assertEqual(self.p.get_postgres_role_from_data_directory(), 'replica')
+        self.assertEqual(self.p.get_postgres_role_from_data_directory(), PostgresqlRole.REPLICA)
 
     @patch('os.remove', Mock())
     @patch('shutil.rmtree', Mock())
@@ -746,33 +748,33 @@ class TestPostgresql(BaseTestPostgresql):
 
     def test_check_for_startup(self):
         with patch('subprocess.call', return_value=0):
-            self.p._state = 'starting'
+            self.p._state = PostgresqlState.STARTING
             self.assertFalse(self.p.check_for_startup())
-            self.assertEqual(self.p.state, 'running')
+            self.assertEqual(self.p.state, PostgresqlState.RUNNING)
 
         with patch('subprocess.call', return_value=1):
-            self.p._state = 'starting'
+            self.p._state = PostgresqlState.STARTING
             self.assertTrue(self.p.check_for_startup())
-            self.assertEqual(self.p.state, 'starting')
+            self.assertEqual(self.p.state, PostgresqlState.STARTING)
 
         with patch('subprocess.call', return_value=2):
-            self.p._state = 'starting'
+            self.p._state = PostgresqlState.STARTING
             self.assertFalse(self.p.check_for_startup())
-            self.assertEqual(self.p.state, 'start failed')
+            self.assertEqual(self.p.state, PostgresqlState.START_FAILED)
 
         with patch('subprocess.call', return_value=0):
-            self.p._state = 'running'
+            self.p._state = PostgresqlState.RUNNING
             self.assertFalse(self.p.check_for_startup())
-            self.assertEqual(self.p.state, 'running')
+            self.assertEqual(self.p.state, PostgresqlState.RUNNING)
 
         with patch('subprocess.call', return_value=127):
-            self.p._state = 'running'
+            self.p._state = PostgresqlState.RUNNING
             self.assertFalse(self.p.check_for_startup())
-            self.assertEqual(self.p.state, 'running')
+            self.assertEqual(self.p.state, PostgresqlState.RUNNING)
 
-            self.p._state = 'starting'
+            self.p._state = PostgresqlState.STARTING
             self.assertFalse(self.p.check_for_startup())
-            self.assertEqual(self.p.state, 'running')
+            self.assertEqual(self.p.state, PostgresqlState.RUNNING)
 
     def test_wait_for_startup(self):
         state = {'sleeps': 0, 'num_rejects': 0, 'final_return': 0}
@@ -795,21 +797,21 @@ class TestPostgresql(BaseTestPostgresql):
             with patch('time.sleep', side_effect=increment_sleeps):
                 self.p.time_in_state = Mock(side_effect=time_in_state)
 
-                self.p._state = 'stopped'
+                self.p._state = PostgresqlState.STOPPED
                 self.assertTrue(self.p.wait_for_startup())
                 self.assertEqual(state['sleeps'], 0)
 
-                self.p._state = 'starting'
+                self.p._state = PostgresqlState.STARTING
                 state['num_rejects'] = 5
                 self.assertTrue(self.p.wait_for_startup())
                 self.assertEqual(state['sleeps'], 5)
 
-                self.p._state = 'starting'
+                self.p._state = PostgresqlState.STARTING
                 state['sleeps'] = 0
                 state['final_return'] = 2
                 self.assertFalse(self.p.wait_for_startup())
 
-                self.p._state = 'starting'
+                self.p._state = PostgresqlState.STARTING
                 state['sleeps'] = 0
                 state['final_return'] = 0
                 self.assertFalse(self.p.wait_for_startup(timeout=2))
@@ -817,7 +819,7 @@ class TestPostgresql(BaseTestPostgresql):
 
         with patch.object(Postgresql, 'check_startup_state_changed', Mock(return_value=False)):
             self.p.cancellable.cancel()
-            self.p._state = 'starting'
+            self.p._state = PostgresqlState.STARTING
             self.assertIsNone(self.p.wait_for_startup())
 
     def test_get_server_parameters(self):
@@ -853,7 +855,8 @@ class TestPostgresql(BaseTestPostgresql):
     def test_get_primary_timeline(self):
         self.assertEqual(self.p.get_primary_timeline(), 1)
 
-    @patch.object(Postgresql, 'get_postgres_role_from_data_directory', Mock(return_value='replica'))
+    @patch.object(Postgresql, 'get_postgres_role_from_data_directory',
+                  Mock(return_value=PostgresqlRole.REPLICA))
     @patch.object(Postgresql, 'is_running', Mock(return_value=False))
     @patch.object(Bootstrap, 'running_custom_bootstrap', PropertyMock(return_value=True))
     @patch('patroni.postgresql.config.logger')
@@ -894,7 +897,7 @@ class TestPostgresql(BaseTestPostgresql):
 
     @patch.object(Postgresql, '_query', Mock(side_effect=RetryFailedError('')))
     def test_received_timeline(self):
-        self.p.set_role('standby_leader')
+        self.p.set_role(PostgresqlRole.STANDBY_LEADER)
         self.p.reset_cluster_info_state(None)
         self.assertRaises(PostgresConnectionException, self.p.received_timeline)
 
@@ -1220,3 +1223,51 @@ class TestPostgresql2(BaseTestPostgresql):
         self.assertEqual(self.p.config.format_dsn(params),
                          'host=1 port=2 sslpassword=pwd sslcrldir=/ gssencmode=prefer channel_binding=prefer '
                          'target_session_attrs=read-write sslnegotiation=postgres')
+
+
+class TestPostgresqlStateMetrics(unittest.TestCase):
+    """Test PostgreSQL state metrics consistency."""
+
+    def test_postgresql_state_metrics_uniqueness(self):
+        """Test that all metrics values are unique."""
+        # Collect all metrics values
+        metrics_values = []
+        for state in PostgresqlState:
+            value = state.index
+            metrics_values.append(value)
+
+        # Check for duplicates
+        unique_values = set(metrics_values)
+        self.assertEqual(len(metrics_values), len(unique_values),
+                         f"Duplicate metrics values found: {metrics_values}")
+
+    def test_postgresql_state_metrics_stability(self):
+        """Test that metrics values are stable and don't change unexpectedly."""
+        # Test specific known values to ensure they don't change
+        expected_values = {
+            PostgresqlState.INITDB: 0,
+            PostgresqlState.INITDB_FAILED: 1,
+            PostgresqlState.CUSTOM_BOOTSTRAP: 2,
+            PostgresqlState.CUSTOM_BOOTSTRAP_FAILED: 3,
+            PostgresqlState.CREATING_REPLICA: 4,
+            PostgresqlState.RUNNING: 5,
+            PostgresqlState.STARTING: 6,
+            PostgresqlState.BOOTSTRAP_STARTING: 7,
+            PostgresqlState.START_FAILED: 8,
+            PostgresqlState.RESTARTING: 9,
+            PostgresqlState.RESTART_FAILED: 10,
+            PostgresqlState.STOPPING: 11,
+            PostgresqlState.STOPPED: 12,
+            PostgresqlState.STOP_FAILED: 13,
+            PostgresqlState.CRASHED: 14,
+        }
+
+        # Iterate over all states to ensure we don't miss any new ones
+        for state in PostgresqlState:
+            with self.subTest(state=state):
+                self.assertIn(state, expected_values,
+                              f"New state {state} added but not included in expected_values test")
+                expected_value = expected_values[state]
+                actual_value = state.index
+                self.assertEqual(actual_value, expected_value,
+                                 f"Metrics value for {state} changed from {expected_value} to {actual_value}")
